@@ -35,7 +35,8 @@ Orthodont$nsexage <- with(Orthodont, nsex*age)
 lmerSlope2 <- lmer(distance ~ age + (0 + age + nsex|Subject), data=Orthodont)
 
 ###############################################
-context("Santize Names")
+#Sanitize Names----
+context("Sanitize Names")
 ################################################
 
 test_that("Sanitize names renames variables in data.frame", {
@@ -50,6 +51,7 @@ test_that("Sanitize names renames variables in data.frame", {
 
 
 ###############################################
+#Strip Attributes----
 context("Strip attributes")
 ################################################
 
@@ -65,6 +67,7 @@ test_that("Attributes can be stripped from data.frame", {
 
 
 ###############################################
+#Random Observation----
 context("Random observation")
 ################################################
 
@@ -111,6 +114,7 @@ test_that("Random observation preserves factor levels", {
 })
 
 ###############################################
+#Collapse frame----
 context("Collapse frame")
 ################################################
 
@@ -139,8 +143,14 @@ context("Subset by a list")
 test_that("Data can be subset by a list", {
   list11 <- list("Sex" = "Male")
   list12 <- list("Sex" = "Male", "Subject" = "M05")
+  list13 <- list("Sex" == "Male")
+  list14 <- list("Sex" == "Male", "Subject" == "M05")
+  list15 <- list("Sex" = "Male", "Subject" == "M05")
   data11 <- merTools:::subsetList(Orthodont, list11)
   data12 <- merTools:::subsetList(Orthodont, list12)
+  expect_error(merTools:::subsetList(Orthodont, list13))
+  expect_error(merTools:::subsetList(Orthodont, list14))
+  expect_error(merTools:::subsetList(Orthodont, list15))
   list21 <- list("YEAR" = "95")
   list22 <- list("LOCATION" = "32", "BROOD" = "503")
   data21 <- merTools:::subsetList(grouseticks, list21)
@@ -156,6 +166,7 @@ test_that("Data can be subset by a list", {
 })
 
 ###############################################
+#Super factor ----
 context("Super factor")
 ################################################
 
@@ -189,6 +200,7 @@ test_that("SuperFactor handles new factor levels correctly", {
 
 
 ###############################################
+#Shuffle----
 context("Shuffle")
 ################################################
 
@@ -200,6 +212,7 @@ test_that("Data can be shuffled", {
 })
 
 ###############################################
+#Find RE Quantiles----
 context("Find RE Quantiles")
 ################################################
 
@@ -227,6 +240,7 @@ test_that("Errors and messages are issued", {
 # })
 
 ###############################################
+#Test observation wiggle----
 context("Test observation wiggle")
 ################################################
 
@@ -306,6 +320,7 @@ test_that("Values are placed correctly", {
 })
 
 ###############################################
+#Test average observation extraction----
 context("Test average observation extraction")
 ################################################
 
@@ -335,9 +350,11 @@ test_that("Subsets work", {
   expect_equal(data1$TICKS, mean(grouseticks$TICKS[grouseticks$YEAR == "97"]))
   expect_equal(data1a$TICKS, mean(grouseticks$TICKS))
   mylist2 <- list("YEAR" = "97", "LOCATION" = "16")
-  data2 <- draw(glmer3LevSlope, type = 'average', varList = mylist2)
+  expect_warning(draw(glmer3LevSlope, type = 'average', varList = mylist2),
+                 "less than 20 rows, averages may be problematic")
   mylist3 <- list("YEAR" = "97", "LOCATION" = c("16", "56"))
-  data3 <- draw(glmer3LevSlope, type = 'average', varList = mylist3)
+  expect_warning(draw(glmer3LevSlope, type = 'average', varList = mylist3),
+                 "fewer than 3 rows, computing global average instead")
 })
 
 test_that("Nested specifications work", {
@@ -355,15 +372,91 @@ test_that("Nested specifications work", {
   data1 <- draw(mod1, "random", varList = mylist2)
   expect_is(data1, "data.frame")
   expect_identical(as.character(data1$order), "Cetacea")
-  data1 <- draw(mod1, "average", varList = mylist1)
+  data1 <- suppressWarnings(draw(mod1, "average", varList = mylist1))
   expect_is(data1, "data.frame")
   expect_identical(as.character(data1$vore), "carni")
-  data1 <- draw(mod1, "average", varList = mylist2)
+  data1 <- suppressWarnings(draw(mod1, "average", varList = mylist2))
   expect_is(data1, "data.frame")
   expect_identical(as.character(data1$order), "Cetacea")
   fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
-  data1 <- draw(fm1, type = "average", varList = list("Subject" = "308"))
+  data1 <- suppressWarnings(draw(fm1, type = "average", varList = list("Subject" = "308")))
   expect_is(data1, "data.frame")
   expect_identical(as.character(data1$Subject), "308")
 })
 
+test_that("findFormFuns works", {
+  #Replicable toy data
+  set.seed(72167)
+  play <- data.frame(
+    a = runif(1000),
+    b = rnorm(1000),
+    c = rbinom(1000, 1, .35),
+    d = rpois(1000, 2)
+  )
+  play$d <- factor(play$d, labels = LETTERS[seq_along(unique(play$d))])
+  play$y <- play$a + 0.5*play$b + 2*play$c -1.8*(play$d=="B") + .43*(play$d == "C") + runif(100, 0, .35)
+  play$grp <- factor(sample(x = paste("Group", 1:43), size = 1000, replace = TRUE))
+  statmode <- function(x){
+    z <- table(as.vector(x))
+    m <- names(z)[z == max(z)]
+    if (length(m) == 1) {
+      return(m)
+    }
+    return(".")
+  }
+  trueMeans <- merTools:::collapseFrame(play)
+  #Estimate toy models
+  ##. Scenario 1: I()
+  s1 <- lmer(y ~ a + b + I(b^2) + c + d + (1|grp), data=play)
+  expect_equal(findFormFuns(s1)[names(trueMeans)], trueMeans)
+  expect_equal(findFormFuns(s1)$b^2, findFormFuns(s1)$`I(b^2)`)
+  expect_length(findFormFuns(s1), 7L)
+
+  ##. Scenario 2: log and no regular a
+  s2 <- lmer(y ~ log(a) + b + c + d + (1|grp), data=play)
+  expect_warning(findFormFuns(s2))
+  expect_false(suppressWarnings(findFormFuns(s2)$`log(a)` == log(trueMeans$a)))
+  expect_silent(findFormFuns(s2, origData = play))
+  expect_equal(findFormFuns(s2, origData = play)$`log(a)`, log(trueMeans$a))
+  ##. Scenario 3: 2 continuous interaction with *
+  s3 <- lmer(y ~ a*b + c + d + (1|grp), data=play)
+  expect_equal(findFormFuns(s3)[names(trueMeans)], trueMeans)
+  expect_length(findFormFuns(s3), 6L)
+  ##. Scenario 4: 2 continuous interaction with :
+  s4 <- lmer(y ~ a:b + c + d + (1|grp), data=play)
+  expect_equal(findFormFuns(s4)[names(trueMeans)], trueMeans)
+  expect_length(findFormFuns(s4), 6L)
+  ##. Scenario 5: 1 cont 1 cat interaction with *
+  s5 <- lmer(y ~ a + c + b*d + (1|grp), data = play)
+  expect_equal(findFormFuns(s5)[names(trueMeans)], trueMeans)
+  expect_length(findFormFuns(s5), 6L)
+  ##. Scenario 6: 1 cont 1 cat interaction with :
+  s6 <- lmer(y ~ a + c + b:d + (1|grp), data = play)
+  expect_equal(findFormFuns(s6)[names(trueMeans)], trueMeans)
+  expect_length(findFormFuns(s6), 6L)
+  ##. Scenario 7: 2 cat interaction with *
+  s7 <- lmer(y ~ a + b + c*d + (1|grp), data = play)
+  expect_equal(findFormFuns(s7)[names(trueMeans)], trueMeans)
+  expect_length(findFormFuns(s7), 6L)
+  ##. Scenario 8: 2 cat interaction with :
+  s8 <- lmer(y ~ a + b + c:d + (1|grp), data = play)
+  expect_equal(findFormFuns(s8)[names(trueMeans)], trueMeans)
+  expect_length(findFormFuns(s8), 6L)
+  ##. Scenario 9: function in random slope
+  s9 <- lmer(y ~ a + b + c + d + (1 + sqrt(abs(b))|grp), data = play)
+  expect_equal(findFormFuns(s9)[names(trueMeans)], trueMeans)
+  expect_equal(findFormFuns(s9)$`sqrt(abs(b))`, sqrt(abs(trueMeans$b)))
+  expect_length(findFormFuns(s9), 7L)
+  ##. Scenario 10: two columns in I with no main effects
+  s10 <- lmer(y ~ I(log(a) + b^3) + c + d + (1|grp), data=play)
+  expect_warning(findFormFuns(s10))
+  expect_false(suppressWarnings(findFormFuns(s10)$`I(log(a) + b^3)`) == log(trueMeans$a) + trueMeans$b^3)
+  expect_silent(findFormFuns(s10, origData = play))
+  expect_equal(findFormFuns(s10, origData = play)$`I(log(a) + b^3)`, log(trueMeans$a) + trueMeans$b^3)
+  ##. Test that draw, draw.merMod and averageObs accept origData and issue warning if appropriate
+  expect_warning(averageObs(s10))
+  expect_silent(averageObs(s10, origData = play))
+  expect_warning(merTools:::draw.merMod(s10, type = "average"))
+  expect_silent(merTools:::draw.merMod(s10, origData = play, type = "average"))
+  expect_silent(merTools:::draw.merMod(s10, type = "random"))
+})
